/**
 * vim: set ts=4 :
 * =============================================================================
 * SourceMod Sample Extension
 * Copyright (C) 2004-2008 AlliedModders LLC.  All rights reserved.
 * =============================================================================
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, version 3.0, as published by the
 * Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * As a special exception, AlliedModders LLC gives you permission to link the
 * code of this program (as well as its derivative works) to "Half-Life 2," the
 * "Source Engine," the "SourcePawn JIT," and any Game MODs that run on software
 * by the Valve Corporation.  You must obey the GNU General Public License in
 * all respects for all other code used.  Additionally, AlliedModders LLC grants
 * this exception to all derivative works.  AlliedModders LLC defines further
 * exceptions, found in LICENSE.txt (as of this writing, version JULY-31-2007),
 * or <http://www.sourcemod.net/license.php>.
 *
 * Version: $Id$
 */

#include "extension.h"

/**
 * @file extension.cpp
 * @brief Implement extension code here.
 */
/*
IGameConfig *g_pGameConf = NULL;

CSteamGameServerAPIContext *g_pSteamAPI = NULL;
ISteamGameServer *g_pSteamGameServer = NULL;
*/
SteamWorks g_Sample;		/**< Global singleton for extension's main interface */

SMEXT_LINK(&g_Sample);

typedef uint32_t Account_t;

IForward *g_pValidateClientForward = NULL;
IForward *g_pCNPCRForward = NULL;

bool SteamWorks::SDK_OnLoad(char *error, size_t maxlength, bool late)
{
	g_pValidateClientForward = forwards->CreateForward("SW_OnValidateClient", ET_Ignore, 2, NULL, Param_Cell, Param_Cell);
	g_pCNPCRForward = forwards->CreateForward("SW_OnNewCompatibilityResult", ET_Ignore, 5, NULL, Param_Cell, Param_Cell, Param_Cell, Param_Cell, Param_Cell);
	return true;
}

void SteamWorks::SDK_OnUnload()
{
	forwards->ReleaseForward(g_pValidateClientForward);
	forwards->ReleaseForward(g_pCNPCRForward);
}

static void NotifyPawn(Account_t parent, Account_t child)
{
	if (g_pValidateClientForward->GetFunctionCount() == 0)
	{
		return;
	}

	g_pValidateClientForward->PushCell(parent);
	g_pValidateClientForward->PushCell(child);
	g_pValidateClientForward->Execute(NULL);
}

void SteamWorks::OnGSClientApprove(GSClientApprove_t *pApprove)
{
	NotifyPawn(pApprove->m_OwnerSteamID.GetAccountID(), pApprove->m_SteamID.GetAccountID());
}

void SteamWorks::OnValidateTicket(ValidateAuthTicketResponse_t *pTicket)
{
	NotifyPawn(pTicket->m_OwnerSteamID.GetAccountID(), pTicket->m_SteamID.GetAccountID());
}

void SteamWorks::OnComputeNewResult(ComputeNewPlayerCompatibilityResult_t *pResult)
{
	if (g_pCNPCRForward->GetFunctionCount() == 0)
	{
		return;
	}
	
	g_pCNPCRForward->PushCell(pResult->m_eResult);
	g_pCNPCRForward->PushCell(pResult->m_cPlayersThatDontLikeCandidate);
	g_pCNPCRForward->PushCell(pResult->m_cPlayersThatCandidateDoesntLike);
	g_pCNPCRForward->PushCell(pResult->m_cClanPlayersThatDontLikeCandidate);
	g_pCNPCRForward->PushCell(pResult->m_SteamIDCandidate.GetAccountID());
	g_pCNPCRForward->Execute(NULL);
}
